/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along with this
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hli@hendriklipka.de
*/
package de.hendriklipka.iwg.jbuilder;

import com.borland.primetime.ide.*;
import com.borland.primetime.node.*;
import com.borland.primetime.vfs.*;
import com.borland.jbuilder.node.*;
import com.borland.jbuilder.*;
import com.borland.jbuilder.jot.*;

import java.util.*;
import java.lang.reflect.Modifier;

import de.hendriklipka.iwg.IJavaCreator;
import de.hendriklipka.iwg.JavaCreatorBase;
import de.hendriklipka.iwg.data.ClassDescriptor;

public class JBuilderCreator extends JavaCreatorBase
{
    protected FileNode _fileNode=null;
    protected JBProject _project=null;

    protected String _className=null;
    protected Hashtable _imports=new Hashtable();
    protected Vector _variables=new Vector();
    protected Vector _interfaces=new Vector();
    protected Vector _methods=new Vector();

    public JBuilderCreator(FileNode jf, JBProject proj)
    {
        _fileNode=jf;
        _project=proj;
        String s="";
        while (s.length()!=INDENT)
            s+=" ";
        _indent=s;
        _2indent=s+s;
    }

    public void setClassName(String className)
    {
        _className=className;
    }

    public void addInterface(String interfaceName)
    {
        addImport(interfaceName);
        // we only add the interface name w/0 package
        int pos=interfaceName.lastIndexOf('.');
        if (-1==pos)
            _interfaces.addElement(interfaceName);
        else
        {
            _interfaces.addElement(interfaceName.substring(pos+1));
        }
    }

    public void addVariable(String varName, String varType)
    {
        _variables.addElement(_indent+"protected "+varType+" "+varName+"=null");
        addImport(varType);
    }

    public void addImport(String importName)
    {
        if ('['==importName.charAt(0))
        {
            importName=importName.substring(2);
        }
        // add only of we have a package name
        // this also skips void and native types
        if (-1!=importName.indexOf('.'))
            _imports.put(importName,importName);
    }

    public void addMethod(String name, String retType, String modifier, String[] body, String[][] parameters)
    {
        addImport(retType);
        String method=_indent+modifier+" "+retType+" "+name+"(";
        for (int i=0;i<parameters[0].length;i++)
        {
            System.out.println("param index="+i+", length="+parameters[0].length+", length="+parameters[1].length);
            if (0!=i)
                method+=",";
            method+=parameters[0][i]+" "+parameters[1][i];
            addImport(parameters[0][i]);
        }
        method+=")\n"+_indent+"{\n";
        for (int i=0;i<body.length;i++)
        {
            method+=_2indent+body[i]+"\n";
        }
        method+=_indent+"}\n";
        _methods.addElement(method);
    }

    public void create(ClassDescriptor classDesc)
    {
        System.out.println("creating source");
        JotPackages jp = _project.getJotPackages();
        JotSourceFile jsf = jp.getSourceFile(_fileNode.getUrl());

        Enumeration imps=_imports.keys();
        while (imps.hasMoreElements())
        {
            String imp=(String)imps.nextElement();
            System.out.println("adding import "+imp);
            jsf.addImport(imp);
        }
        jp.commit(jsf);
        System.out.println("commited");

        try{
          _fileNode.save();
            System.out.println("saved");
        }

        catch(java.io.IOException ex){
          ex.printStackTrace();
        }
        jp.release(jsf);
    }
}
/*
    JotClassSource jcs = jsf.addClass(null, false, fileName, false);
    jcs.setModifiers(Modifier.PUBLIC);
    jcs.setSuperclass("JPanel");

    JotFieldDeclaration jfd = jcs.addField(null, false, "int", "ctr");
    jfd.setModifiers(Modifier.PRIVATE | Modifier.STATIC);

    JotConstructorSource xtor = jcs.addConstructor(null, false);
    JotCodeBlock xtorBlock = xtor.getCodeBlock();
    xtorBlock.addAssignment(null, false, "this.ctr", "4");

    JotMethodSource getterSource = jcs.addMethod(null, false, "int", "getCtr");
    getterSource.setModifiers(Modifier.PUBLIC);
    getterSource.getCodeBlock().addReturnStatement(null, false, "this.ctr");

    JotMethodSource setterSource = jcs.addMethod(null, false, "void", "setCtr");
    setterSource.setModifiers(Modifier.PUBLIC);
    setterSource.addParameter(null, false, "int", "x");
    setterSource.getCodeBlock().addAssignment(null, false, "this.ctr", "x");

    // now we add the main but add it in between the constructor and the getter
    // this could be done by using xtor as the marker and setting "before" to false
    // or as shown by using getterSource as the marker and setting "before" to true
    JotMethodSource mainSource = jcs.addMethod(getterSource, true, "void", "main");
    mainSource.addParameter(null, false, "String[]", "argv");
    mainSource.setModifiers(Modifier.STATIC);
    JotCodeBlock mainBlock = mainSource.getCodeBlock();
    JotVariableDeclaration jvd = mainBlock.addVariableDeclaration(null, false, "JotWriting", "jw");
    jvd.setInitializer("new JotWriting()");
    JotVariableDeclaration jvd2 = mainBlock.addVariableDeclaration(jvd, false, "int", "loopCtr");
    jvd2.setInitializer("jw.getCtr()");

    // add the for loop
    JotFor forLoop = mainBlock.addForStatement(null, false, "int i = 0; i < loopCtr; i++");
    forLoop.getCodeBlock().addStatement(null, false, true, "System.out.println(\"JBuilder Rocks!\")");

    mainBlock.getCodeBlock().addBlankLine(forLoop, true);

    jp.commit(jsf);

    try{
      jf.save();
    }

    catch(IOException ex){
      ex.printStackTrace();
    }
    jp.release(jsf);
  }

*/
